# -*- coding: utf-8 -*-
#
# Pyserini: Python interface to the Anserini IR toolkit built on Lucene
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

'''
Module for providing python interface to Anserini searchers
'''

from ..pyclass import JSearcher, JString, JArrayList

import logging
logger = logging.getLogger(__name__)

class SimpleSearcher:
    '''
    Wrapper class for Anserini's SimpleSearcher.
            
    Parameters
    ----------
    index_dir : str
        Path to Lucene index directory
    '''
    
    def __init__(self, index_dir):
        self.object = JSearcher(JString(index_dir))
    
    def search(self, q, k=10, t=-1):
        '''
        Parameters
        ----------
        q : str
            Query string
        k : int
            Number of hits to return
        t : int
            Query tweet time for searching tweets  
            
        Returns
        -------
        results : list of io.anserini.search.SimpleSearcher$Result
            List of document hits returned from search
        '''
        return self.object.search(JString(q), k, t)
        
    def batch_search(self, queries, qids, k=10, t=-1, threads=1):
        '''
        Parameters
        ----------
        queries : list of str
            list of query strings
        qids : list of str
            list of corresponding query ids
        k : int
            Number of hits to return
        t : int
            Query tweet time for searching tweets  
        threads : int
            Maximum number of threads 
            
        Returns
        -------
        result_dict : dict of {str : io.anserini.search.SimpleSearcher$Result}
            Dictionary of {qid : document hits} returned from each query
        '''
        query_strings = JArrayList()
        qid_strings = JArrayList()
        for query in queries:
            jq = JString(query.encode('utf8'))
            query_strings.add(jq)

        for qid in qids:
            jqid = JString(qid)
            qid_strings.add(jqid)

        results = self.object.batchSearch(query_strings, qid_strings, int(k), int(t), int(threads)).entrySet().toArray()
        return {r.getKey() : r.getValue() for r in results}

    def search_fields(self, q, f, boost, k):
        '''
        Parameters
        ----------
        q : str
            Query string
        f : str
            Name of additional field to search over
        boost : float
            Weight boost for additional field
        k : int
            Number of hits to return
            
        Returns
        -------
        results : list of io.anserini.search.SimpleSearcher$Result
            List of document hits returned from search
        '''
        return self.object.searchFields(JString(q), JString(f), float(boost), k)
    
    def set_search_tweets(self, flag):
        '''
        Parameters
        ----------
        flag : bool
            True if searching over tweets
        '''
        self.object.setSearchTweets(flag)
        
    def set_rm3_reranker(self, fb_terms=10, fb_docs=10, 
                         original_query_weight=float(0.5), 
                         rm3_output_query=False):
        '''
        Parameters
        ----------
        fb_terms : int
            RM3 parameter for number of expansion terms
        fb_docs : int
            RM3 parameter for number of documents
        original_query_weight : float
            RM3 parameter for weight to assign to the original query
        rm3_output_query : bool
            True if we want to print original and expanded queries for RM3
        '''
        self.object.setRM3Reranker(fb_terms, fb_docs, 
                                   original_query_weight, rm3_output_query)
        
    def set_default_reranker(self):
        self.object.setDefaultReranker()
        
    def set_lm_dirichlet_similarity(self, mu):
        '''
        Parameters
        ----------
        mu : float
            Dirichlet smoothing parameter
        '''
        self.object.setLMDirichletSimilarity(float(mu))
        
    def set_lm_jelinek_mercer_similarity(self, lam):
        '''
        Parameters
        ----------
        lam : float
            Jelinek Mercer smoothing parameter
        '''
        self.object.set.LMJelinekMercerSimilarity(float(lam))
        
    def set_bm25_similarity(self, k1, b):
        '''
        Parameters
        ----------
        k1 : float
            BM25 k1 parameter
        b : float
            BM25 b parameter
        '''
        self.object.setBM25Similarity(float(k1), float(b))
        
    def set_dfr_similarity(self, c):
        '''
        Parameters
        ----------
        c : float
            DFR c parameter
        '''
        self.object.setDFRSimilarity(float(c))
            
    def set_ib_similarity(self, c):
        '''
        Parameters
        ----------
        c : float
            Information-based c parameter
        '''
        self.object.setIBSimilarity(float(c))

    def set_f2exp_similarity(self, s):
        '''
        Parameters
        ----------
        s : float
            F2Exp s parameter
        '''
        self.object.setF2ExpSimilarity(float(s))
        
    def set_f2log_similarity(self, s):
        '''
        Parameters
        ----------
        s : float
            F2Log s parameter
        '''
        self.object.setF2LogSimilarity(float(s))

    def doc(self, ldocid):
        '''
        Parameters
        ----------
        ldocid : int
            Internal Lucene docid of a document
        
        Returns
        -------
        result : str
            Raw content of the given document
        
        '''
        return self.object.doc(ldocid)
        
    def close(self):
        self.object.close()



# -*- coding: utf-8 -*-
#
# Pyserini: Python interface to the Anserini IR toolkit built on Lucene
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from ..pyclass import JCollections, JPaths, cast
from ..multithreading import Counters
import re

import logging
logger = logging.getLogger(__name__)

class Collection:
    '''
    Iterable wrapper class for Anserini's DocumentCollection.
            
    Parameters
    ----------
    collection_class : str
        Name of collection class to instantiate
    collection_path : str
        Path to directory containing collection
    '''    
    def __init__(self, collection_class, collection_path):
        self.counters = Counters()
        self.collection_class = collection_class
        self.collection_path = JPaths.get(collection_path)
        self.object = self._get_collection()     
        self.object.setCollectionPath(self.collection_path)
        self.collection_iterator = self.object.iterator()
        
    def _get_collection(self):
        try:
            return JCollections[self.collection_class].value()
        except:
            raise ValueError(self.collection_class)
            
    def __iter__(self):
        return self

    def __next__(self):
        if self.collection_iterator.hasNext():
            fs = self.collection_iterator.next()
            return FileSegment(self, fs, fs.getSegmentPath())
        else:
            raise StopIteration
                
            
class FileSegment:
    '''
    Iterable wrapper class for Anserini's FileSegment.
            
    Parameters
    ----------
    collection : Collection
        Parent collection of the file segment
    segment : io.anserini.collection.FileSegment
        FileSegment object to create wrapper from
    segment_path : str
        Path to file backing the file segment
    '''  
    def __init__(self, collection, segment, segment_path):
        self.collection = collection
        try:
            self.object = cast(collection.object.getClass().getName() + 
                                '$Segment', segment)
        except:
            logger.exception('Exception from casting FileSegment type...')
            self.object = cast('io.anserini.collection.FileSegment', 
                                segment)
            
        self.segment_iterator = self.object.iterator()
        self.segment_path = segment_path
        self.segment_name = re.sub(r'\\|\/', '-', 
                                   collection.collection_path.relativize(
                                           segment_path).toString())
        
    def __iter__(self):
        return self

    def __next__(self):
        if self.object.iterator().hasNext():
            d = self.object.iterator().next()
            return Document(self, d)
        else:
            # log if iteration stopped by error
            if (self.object.getErrorStatus()):
                logger.error(self.segment_name + 
                             ': Error from segment iteration, stopping...')
                self.collection.counters.errors.increment()

            # stop iteration and log skipped documents
            skipped = self.object.getSkippedCount()
            if (skipped > 0):
                self.collection.counters.skips.increment(skipped)
                logger.warn(self.segment_name + 
                                 ': ' + str(skipped) + ' documents skipped')
            self.object.close()
            raise StopIteration

        
class Document:  
    '''
    Wrapper class for Anserini's SourceDocument.
            
    Parameters
    ----------

    segment : FileSegment
        Parent segment of the source document
    document : io.anserini.collection.SourceDocument
        SourceDocument object to create wrapper from
    ''' 
    def __init__(self, segment, document):
        self.segment = segment
        self.object = document
        self.id = self.object.id()
        self.indexable = self.object.indexable()
        try:
            self.contents = self.object.content()
        except:
            self.contents = self.object.getContent()
        



# -*- coding: utf-8 -*-
#
# Pyserini: Python interface to the Anserini IR toolkit built on Lucene
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

'''
Module for hiding Python-Java calls via Pyjnius
'''

### Pyjnius setup

from .setup import configure_classpath, os
# If the environment variable isn't defined, look in the current directory.
configure_classpath(os.environ['ANSERINI_CLASSPATH'] if 'ANSERINI_CLASSPATH' in os.environ else '.')
    
from jnius import autoclass, cast
from enum import Enum

### Java

JString = autoclass('java.lang.String')
JPath = autoclass('java.nio.file.Path')
JPaths = autoclass('java.nio.file.Paths')
JList = autoclass('java.util.List')
JArrayList = autoclass('java.util.ArrayList')

### Search

JSearcher = autoclass('io.anserini.search.SimpleSearcher')

### Generator

class JIndexHelpers:
    
    def JArgs():
        args = autoclass('io.anserini.index.IndexCollection$Args')()
        # See https://github.com/castorini/anserini/pull/831/files
        # args = autoclass('io.anserini.index.IndexArgs')()
        args.storeRawDocs = True ## to store raw text as an option
        args.dryRun = True ## So that indexing will be skipped
        return args
    
    def JCounters():
        IndexCollection = autoclass('io.anserini.index.IndexCollection')
        Counters = autoclass('io.anserini.index.IndexCollection$Counters')
        return Counters(IndexCollection)

class JGenerators(Enum):        
    LuceneDocumentGenerator = autoclass('io.anserini.index.generator.LuceneDocumentGenerator')
    JsoupGenerator = autoclass('io.anserini.index.generator.JsoupGenerator')
    TweetGenerator = autoclass('io.anserini.index.generator.TweetGenerator')
    WapoGenerator = autoclass('io.anserini.index.generator.WapoGenerator')

### Collection
        
class JCollections(Enum):
    CarCollection = autoclass('io.anserini.collection.CarCollection')
    ClueWeb09Collection = autoclass('io.anserini.collection.ClueWeb09Collection')
    ClueWeb12Collection = autoclass('io.anserini.collection.ClueWeb12Collection')
    HtmlCollection = autoclass('io.anserini.collection.HtmlCollection')
    JsonCollection = autoclass('io.anserini.collection.JsonCollection')
    NewYorkTimesCollection = autoclass('io.anserini.collection.NewYorkTimesCollection')
    TrecCollection = autoclass('io.anserini.collection.TrecCollection')
    TrecwebCollection = autoclass('io.anserini.collection.TrecwebCollection')
    TweetCollection = autoclass('io.anserini.collection.TweetCollection')
    WashingtonPostCollection = autoclass('io.anserini.collection.WashingtonPostCollection')
    WikipediaCollection = autoclass('io.anserini.collection.WikipediaCollection')




# -*- coding: utf-8 -*-
#
# Pyserini: Python interface to the Anserini IR toolkit built on Lucene
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import threading


class ThreadSafeCount:
    
    def __init__(self):
        self.value = 0
        self.lock = threading.Lock()
        
    def increment(self, inc=1):
        with self.lock:
            self.value += inc
            return self.value
     
            
class Counters:
    
    def __init__(self):
        self.indexable = ThreadSafeCount()
        self.unindexable = ThreadSafeCount()
        self.skipped = ThreadSafeCount()
        self.errors = ThreadSafeCount()




import setuptools

with open("project-description.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="pyserini",
    version="0.6.0.0",
    author="Jimmy Lin",
    author_email="jimmylin@uwaterloo.ca",
    description="Python interface to the Anserini IR toolkit built on Lucene",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/castorini/pyserini",
    install_requires=['Cython', 'pyjnius'],
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
)



# -*- coding: utf-8 -*-
#
# Pyserini: Python interface to the Anserini IR toolkit built on Lucene
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

'''
Module for adding Anserini jar to classpath for pyjnius usage
'''

import os
import glob
import jnius_config

def configure_classpath(anserini_root="."):   
    ''' 
    Parameters
    ----------
    anserini_root : str
        (Optional) path to root anserini directory.
    
    '''
    paths = glob.glob(os.path.join(anserini_root, 'anserini-*-fatjar.jar'))
    if not paths:
        raise Exception('No matching jar file found in {}'.format(os.path.abspath(anserini_root)))

    latest = max(paths, key=os.path.getctime)
    jnius_config.set_classpath(latest)
    



# -*- coding: utf-8 -*-
#
# Pyserini: Python interface to the Anserini IR toolkit built on Lucene
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

'''
Module for providing python interface to Anserini generators
'''

from ..pyclass import JIndexHelpers, JGenerators

import logging
logger = logging.getLogger(__name__)

class Generator:
    '''
    Wrapper class for Anserini's generators.
            
    Parameters
    ----------
    generator_class : str
        Name of generator class to instantiate
    '''
        
    def __init__(self, generator_class):
        self.counters = JIndexHelpers.JCounters()
        self.args = JIndexHelpers.JArgs()
        self.generator_class = generator_class
        self.object = self._get_generator()
    
    def _get_generator(self):
        try:
            return JGenerators[self.generator_class].value(self.args, self.counters)
        except:
            raise ValueError(self.generator_class)
            
    def create_document(self, document):
        '''
        Parameters
        ----------
        document : pyserini.collection.pycollection.Document
            Collection document to create Lucene document from
            
        Returns
        -------
        result : org.apache.lucene.document.Document
            Lucene document generated
        '''
        return self.object.createDocument(document.object)
            
